#!/usr/local/bin/perl
#
# glossExtract.pl ver 0.03
#
# Program to extract glosses from WordNet and print them out to the
# standard output device.
#
# Copyright (c) 2002-2004
#
# Siddharth Patwardhan, University of Utah
# sidd at cs.utah.edu
#
# Ted Pedersen, University of Minnesota Duluth
# tpederse at d.umn.edu
#
# Jason Michelizzi, University of Minnesota Duluth
# mich0212 at d.umn.edu
#
# Alexander R. Pruss, Georgetown University
# ap85 at georgetown.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to:
#
#    The Free Software Foundation, Inc.,
#    59 Temple Place - Suite 330,
#    Boston, MA  02111-1307, USA.
#
#
#------------------------------------------------------------------------

use strict;
use warnings;

use Getopt::Long;
use Search::Dict;

# we don't use BerkeleyDB anymore
#use BerkeleyDB;

# Declarations!
my $wnUnixPath;     # Path to WordNet on a unix system.
my $wnPCPath;       # Path to WordNet on a Windows system.
my $line;           # One line of text.

# we don't use these anymore
#my @parts;          # Parts of a line.
#my @words;          # Words in the line.
#my %compounds;      # List of compounds.
#my %stopWords;      # List of stop words.
#my %dbHash;         # Hash linked to the database.
#my %wordMatrix;     # Word Vectors.
#my %wordList;       # List of words.

# JM 1-5-04
# added support for outputting offsets or wps

our ($opt_offsets, $opt_wps, $opt_version, $opt_help);
our ($opt_wnpath, $opt_noexamples, $opt_keepcase);

# Now get the options!
my $result = GetOptions("version", "help", "wnpath=s", "noexamples",
			"offsets", "wps", "keepcase");

unless ($result) {
  printUsage();
  exit (1);
}

# If the version information has been requested
if(defined $opt_version)
{
    $opt_version = 1;
    &printVersion();
    exit;
}

# If detailed help has been requested
if(defined $opt_help)
{
    $opt_help = 1;
    &printHelp();
    exit;
}

# Hack to prevent "single occurrence of variable" warning
$opt_noexamples = 1 if(defined $opt_noexamples);

# Check if path to WordNet Data files has been provided ... If so ... save it.
if(defined $opt_wnpath)
{
    $wnPCPath = $opt_wnpath;
    $wnUnixPath = $opt_wnpath;
}
else
{
    # JM 1-6-04
    # Modified following to use WordNet 2.0 by default
    $wnPCPath = (defined $ENV{"WNHOME"})
      ? $ENV{"WNHOME"} : "C:\\Program Files\\WordNet\\2.0";
    $wnUnixPath = (defined $ENV{"WNHOME"})
      ? $ENV{"WNHOME"} : "/usr/local/WordNet-2.0";
    $wnPCPath = (defined $ENV{"WNSEARCHDIR"})
      ? $ENV{"WNSEARCHDIR"} : $wnPCPath."\\dict";
    $wnUnixPath = (defined $ENV{"WNSEARCHDIR"})
      ? $ENV{"WNSEARCHDIR"} : $wnUnixPath."/dict";
}

# Opening WordNet data files and reading the glosses...
open(NDAT, $wnUnixPath."/data.noun") || open(NDAT, $wnPCPath."\\noun.dat")
  or die "Unable to open data file: $!";
open(VDAT, $wnUnixPath."/data.verb") || open(VDAT, $wnPCPath."\\verb.dat")
  or die "Unable to open data file: $!";
open(ADAT, $wnUnixPath."/data.adj") || open(ADAT, $wnPCPath."\\adj.dat")
  or die "Unable to open data file: $!";
open(RDAT, $wnUnixPath."/data.adv") || open(RDAT, $wnPCPath."\\adv.dat")
  or die "Unable to open data file: $!";

if ($opt_wps) {
    open(NIDX, $wnUnixPath."/index.noun") || open(NIDX, "$wnPCPath\\noun.idx")
      or die "Unable to open index file: $!";
    open(VIDX, $wnUnixPath."/index.verb") || open(VIDX, "$wnPCPath\\verb.idx")
      or die "Unable to open index file: $!";
    open(AIDX, $wnUnixPath."/index.adj") || open(AIDX, "$wnPCPath\\adj.idx")
      or die "Unable to open index file: $!";
    open(RIDX, $wnUnixPath."/index.adv") || open(RIDX, "$wnPCPath\\adv.idx")
      or die "Unable to open index file: $!";
}

foreach my $fh (*RDAT, *NDAT, *VDAT, *ADAT)
{
    foreach(1 .. 29)
    {
	$line = <$fh>;
    }
    while($line = <$fh>)
    {
        $line =~ m/^(\d\d\d\d\d\d\d\d)/
	  or die "Formatting error in WordNet data file";

	my ($offset, undef, $pos, $wcnt, @tokens) = split /\s+/, $line;
	my @words = ();
	for (1 .. hex ($wcnt)) {
	  push (@words, shift (@tokens));
	  shift @tokens;
	}
	
	$line =~ s/[\r\f\n]//g;
	$line =~ s/.*\|//;
	$line =~ s/\".*\"//g if(defined $opt_noexamples);
	$line =~ s/^\s*//;
	$line =~ s/\s*$//;

	if ($opt_offsets) {
	  print "$offset $pos ";
	}
	elsif ($opt_wps) {
	  my $wps = getWps ($offset, $pos, @words);
	  print "$wps| ";
	}
	print "$line\n";

    }
}

close (NDAT);
close (VDAT);
close (ADAT);
close (RDAT);

if ($opt_wps) {
    close (NIDX);
    close (VIDX);
    close (AIDX);
    close (RIDX);
}

# ----------------- Subroutines start Here ----------------------

# Subroutine to print detailed help
sub printHelp
{
    printUsage();
    print "\nThis program extracts all glosses from WordNet and prints them out to\n";
    print "the standard output device.\n";
    print "Options: \n";
    print "--wnpath         WNPATH specifies the path of the WordNet data files.\n";
    print "                 Ordinarily, this path is determined from the \$WNHOME\n";
    print "                 environment variable. But this option overides this\n";
    print "                 behavior.\n";
    print "--wps            Show the word senses the associated with each gloss\n";
    print "--keepcase       When used with --wps, maintains the original case and\n";
    print "                 syntactic markers from WordNet.\n";
    print "--offsets        Show the offset of the synset associated with each gloss.\n";
    print "--noexamples     Removes examples from the glosses.\n";
    print "--keepcase       Do not lowercase all words.\n";
    print "--help           Displays this help screen.\n";
    print "--version        Displays version information.\n\n";
}

# Subroutine to print minimal usage notes
sub minimalUsageNotes
{
    printUsage();
    print "Type glossExtract.pl --help for detailed help.\n";
}

# Subroutine that prints the usage
sub printUsage
{
    print "Usage: glossExtract.pl [{ --wnpath WNPATH --noexamples | [--wps [--keepcase] | --offsets] | --help | --version }]\n"
}

# Subroutine to print the version information
sub printVersion
{
    print "glossExtract.pl version 0.03\n";
    print "Copyright (c) 2002-2004,\nSiddharth Patwardhan, Ted Pedersen, Jason Michelizzi, and Alexander Pruss.\n\n";
    print "glossExtract.pl comes with ABSOLUTELY NO WARRANTY.  This is free software,\n";
    print "and you are welcome to redistribute it under certain conditions.  Please\n";
    print "read the file gpl.txt that came with this program.\n";
}

sub lower ($)
{
    my $s = shift;
    $s =~ tr/A-Z /a-z_/;
    $s =~ s/\(.*\)$//;
    return $s;
}

sub getWps
{
    my ($offset, $pos, @words) = @_;
    my $str = "";
    my $fh;

    if ($pos eq 'n') {
      $fh = *NIDX;
    }
    elsif ($pos eq 'v') {
      $fh = *VIDX;
    }
    elsif (($pos eq 'a') or ($pos eq 's')) {
      $fh = *AIDX;
    }
    elsif ($pos eq 'r') {
      $fh = *RIDX;
    }
    else {
      die "Fatal error: unknown part of speech '$pos'";
    }

  WORD:
    foreach my $word (@words) {
        my $lowerword = lower $word;
        if (! $opt_keepcase) {
          $word = $lowerword;
        }
        seek $fh, 0, 0;
        my $result = look ($fh, $lowerword);
        if ($result < 0) {
	  warn "Couldn't find $lowerword in WordNet";
	  next WORD;
	}
	my $line;

      READ_FH:
	while (<$fh>) {
	  $line = $_;
	  last READ_FH if m/^$lowerword /;
	}
	my ($w, $p, $scnt, $pcnt, @tokens) = split /\s+/, $line;

	for (1..$pcnt) {
	  shift @tokens;
	}
	shift @tokens;
	shift @tokens;

	my $found = 0;
	foreach my $i (0..$#tokens) {
	  if ($tokens[$i] eq $offset) {
	    $str .= "$word#$pos#".($i+1)." ";
	    $found = 1;
	  }
	}
	unless ($found) {
	  warn "Couldn't find $offset ($word) in the list ".
	    join (", ", @tokens);
	}
    }
    return $str;
}

__END__
